package cn.com.infosec.crypto.digests;

import cn.com.infosec.crypto.ExtendedDigest;

public class MD2Digest implements ExtendedDigest {
   private static final int DIGEST_LENGTH = 16;
   private static final byte[] S = new byte[]{41, 46, 67, -55, -94, -40, 124, 1, 61, 54, 84, -95, -20, -16, 6, 19, 98, -89, 5, -13, -64, -57, 115, -116, -104, -109, 43, -39, -68, 76, -126, -54, 30, -101, 87, 60, -3, -44, -32, 22, 103, 66, 111, 24, -118, 23, -27, 18, -66, 78, -60, -42, -38, -98, -34, 73, -96, -5, -11, -114, -69, 47, -18, 122, -87, 104, 121, -111, 21, -78, 7, 63, -108, -62, 16, -119, 11, 34, 95, 33, -128, 127, 93, -102, 90, -112, 50, 39, 53, 62, -52, -25, -65, -9, -105, 3, -1, 25, 48, -77, 72, -91, -75, -47, -41, 94, -110, 42, -84, 86, -86, -58, 79, -72, 56, -46, -106, -92, 125, -74, 118, -4, 107, -30, -100, 116, 4, -15, 69, -99, 112, 89, 100, 113, -121, 32, -122, 91, -49, 101, -26, 45, -88, 2, 27, 96, 37, -83, -82, -80, -71, -10, 28, 70, 97, 105, 52, 64, 126, 15, 85, 71, -93, 35, -35, 81, -81, 58, -61, 92, -7, -50, -70, -59, -22, 38, 44, 83, 13, 110, -123, 40, -124, 9, -45, -33, -51, -12, 65, -127, 77, 82, 106, -36, 55, -56, 108, -63, -85, -6, 36, -31, 123, 8, 12, -67, -79, 74, 120, -120, -107, -117, -29, 99, -24, 109, -23, -53, -43, -2, 59, 0, 29, 57, -14, -17, -73, 14, 102, 88, -48, -28, -90, 119, 114, -8, -21, 117, 75, 10, 49, 68, 80, -76, -113, -19, 31, 26, -37, -103, -115, 51, -97, 17, -125, 20};
   private byte[] X;
   private int xOff;
   private byte[] M;
   private int mOff;
   private byte[] C;
   private int COff;

   public MD2Digest() {
      this.X = new byte[48];
      this.M = new byte[16];
      this.C = new byte[16];
      this.reset();
   }

   public MD2Digest(MD2Digest t) {
      MD2Digest var10000 = this;
      MD2Digest var10001 = t;
      MD2Digest var10002 = t;
      MD2Digest var10004 = t;
      MD2Digest var10005 = t;
      MD2Digest var10007 = t;
      MD2Digest var10008 = t;
      super();
      byte[] t1;
      this.X = t1 = new byte[48];
      this.M = new byte[16];
      this.C = new byte[16];
      byte[] var7 = var10008.X;
      int var2 = var7.length;
      System.arraycopy(var7, 0, t1, 0, var2);
      this.xOff = var10007.xOff;
      byte[] var3 = var10005.M;
      t1 = this.M;
      var2 = var3.length;
      System.arraycopy(var3, 0, t1, 0, var2);
      this.mOff = var10004.mOff;
      byte[] var8 = var10002.C;
      byte[] var4 = this.C;
      int t2 = var8.length;
      System.arraycopy(var8, 0, var4, 0, t2);
      var10000.COff = var10001.COff;
   }

   public String getAlgorithmName() {
      return "MD2";
   }

   public int getDigestSize() {
      return 16;
   }

   public int doFinal(byte[] out, int outOff) {
      int var3;
      byte[] var5;
      for(byte var4 = (byte)(this.M.length - (var3 = this.mOff)); var3 < (var5 = this.M).length; ++var3) {
         var5[var3] = var4;
      }

      this.processCheckSum(var5);
      this.processBlock(this.M);
      this.processBlock(this.C);
      System.arraycopy(this.X, this.xOff, out, outOff, 16);
      this.reset();
      return 16;
   }

   public void reset() {
      this.xOff = 0;

      int var1;
      byte[] var2;
      for(var1 = 0; var1 != (var2 = this.X).length; ++var1) {
         var2[var1] = 0;
      }

      this.mOff = 0;

      for(var1 = 0; var1 != (var2 = this.M).length; ++var1) {
         var2[var1] = 0;
      }

      this.COff = 0;

      for(var1 = 0; var1 != (var2 = this.C).length; ++var1) {
         var2[var1] = 0;
      }

   }

   public void update(byte in) {
      byte[] var2 = this.M;
      int var3;
      int var4;
      int var10000 = var4 = (var3 = this.mOff) + 1;
      this.mOff = var4;
      var2[var3] = in;
      if (var10000 == 16) {
         this.processCheckSum(var2);
         this.processBlock(this.M);
         this.mOff = 0;
      }

   }

   public void update(byte[] in, int inOff, int len) {
      while(this.mOff != 0 && len > 0) {
         this.update(in[inOff]);
         ++inOff;
         --len;
      }

      while(len > 16) {
         System.arraycopy(in, inOff, this.M, 0, 16);
         this.processCheckSum(this.M);
         this.processBlock(this.M);
         len -= 16;
         inOff += 16;
      }

      while(len > 0) {
         this.update(in[inOff]);
         ++inOff;
         --len;
      }

   }

   protected void processCheckSum(byte[] m) {
      byte var2 = this.C[15];

      for(int var3 = 0; var3 < 16; ++var3) {
         byte[] var4;
         byte[] var10000 = var4 = this.C;
         var4[var3] ^= S[(m[var3] ^ var2) & 255];
         var2 = var10000[var3];
      }

   }

   protected void processBlock(byte[] m) {
      int var2;
      for(var2 = 0; var2 < 16; ++var2) {
         byte[] var3;
         byte[] var10000 = var3 = this.X;
         int var4 = var2 + 16;
         var3[var4] = m[var2];
         var4 = var2 + 32;
         var10000[var4] = (byte)(m[var2] ^ var3[var2]);
      }

      int m1 = 0;
      byte var7 = 0;

      for(int var8 = var7; var8 < 18; var8 = var2) {
         for(var2 = 0; var2 < 48; ++var2) {
            byte m2;
            byte[] var9;
            byte var10 = m2 = (byte)((var9 = this.X)[var2] ^ S[m1]);
            var9[var2] = m2;
            m1 = var10 & 255;
         }

         m1 = (m1 + var8) % 256;
         var2 = var8 + 1;
      }

   }

   public int getByteLength() {
      return 16;
   }
}
